<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://etracker.com
 * @since      1.0.0
 *
 * @package    Etracker
 * @subpackage Etracker/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Etracker
 * @subpackage Etracker/admin
 * @author     etracker GmbH <support@etracker.com>
 */
class Etracker_Admin {
	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of this plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	/**
	 * Register plugin settings page.
	 *
	 * @since   1.0.0
	 */
	public function add_admin_menu() {
		add_options_page( 'etracker', 'etracker Analytics', 'manage_options', 'etracker', array( $this, 'options_page' ) );
	}

	/**
	 * Admin page with plugin settings.
	 *
	 * @since    1.0.0
	 */
	public function options_page() {
		// check user capabilities.
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		?>
			<form action='options.php' method='post'>

				<?php
				settings_fields( 'pluginPage' );
				do_settings_sections( 'pluginPage' );
				submit_button();
				?>

			</form>
			<?php
	}

	/**
	 * Register admin page settins for this plugin.
	 *
	 * @since       1.0.0
	 */
	public function settings_init() {
		register_setting(
			'pluginPage',
			'etracker_settings',
			array(
				'sanitize_callback' => array( $this, 'validate_options' ),
				'default'           => array(
					'etracker_block_cookies' => 'true',
				),
			)
		);

		add_settings_section(
			'etracker_pluginPage_section',
			__( 'etracker Analytics', 'etracker' ),
			array( $this, 'settings_section_callback' ),
			'pluginPage'
		);

		add_settings_field(
			'etracker_secure_code',
			__( 'Account Key', 'etracker' ),
			array( $this, 'render_secure_code' ),
			'pluginPage',
			'etracker_pluginPage_section'
		);

		add_settings_field(
			'etracker_block_cookies',
			__( 'Cookie-less tracking', 'etracker' ),
			array( $this, 'render_block_cookies' ),
			'pluginPage',
			'etracker_pluginPage_section'
		);

		add_settings_field(
			'etracker_disable_et_pagename',
			__( 'Disable et_pagename', 'etracker' ),
			array( $this, 'render_disable_et_pagename' ),
			'pluginPage',
			'etracker_pluginPage_section'
		);

		add_settings_field(
			'etracker_signalize_ready',
			__( 'Signalize ready', 'etracker' ),
			array( $this, 'render_signalize_ready' ),
			'pluginPage',
			'etracker_pluginPage_section'
		);

		add_settings_field(
			'etracker_custom_attributes',
			__( 'Custom attributes', 'etracker' ),
			array( $this, 'render_custom_attributes' ),
			'pluginPage',
			'etracker_pluginPage_section'
		);
	}

	/**
	 * Validate setting values before processing them.
	 *
	 * @param   array $options Options submitted by settings form.
	 * @return  array Validated options
	 * @since   1.0.0
	 */
	public function validate_options( $options ) {
		$validated                                 = array();
		$validated['etracker_block_cookies']       = ( 'true' == $options['etracker_block_cookies'] ) ? 'true' : 'false';
		$validated['etracker_secure_code']         = $options['etracker_secure_code'];
		$validated['etracker_disable_et_pagename'] = ( 'true' == $options['etracker_disable_et_pagename'] ) ? 'true' : 'false';
		$validated['etracker_custom_attributes']            = $options['etracker_custom_attributes'];

		return $validated;
	}

	/**
	 * Admin page settings description callback.
	 *
	 * @since   1.0.0
	 */
	public function settings_section_callback() {
		esc_html_e( 'This plugin enables etracker Analytics for your entire WordPress site.', 'etracker' );
		?>
		<p><?php esc_html_e( 'How to use this plugin', 'etracker' ); ?>:</p><ol>
				<li><?php esc_html_e( 'Enter your etracker Account Key', 'etracker' ); ?></li>
				<li><?php esc_html_e( 'Choose either to enable or disable cookies by default', 'etracker' ); ?></li>
				<li><?php esc_html_e( 'Save changes', 'etracker' ); ?></li>
		</ol>
		<?php
	}

	/**
	 * Render secure code settings form part for admin page.
	 *
	 * @since       1.0.0
	 */
	public function render_secure_code() {
		$options = get_option( 'etracker_settings' );
		?>
		<input type='text' name='etracker_settings[etracker_secure_code]' value='<?php echo esc_attr( $options['etracker_secure_code'] ); ?>'>
		<p class="description">
		<?php esc_html_e( 'Activate plugin by entering your individual Account Key.', 'etracker' ); ?>
		<a href="https://www.etracker.com/signup?etcc_cmp=eA%20Plugin&etcc_med=Pluginstore&etcc_grp=wordpress&etcc_ctv=pluginsettings" target="_blank"><?php esc_html_e( 'Signup for free.', 'etracker' ); ?></a>
		</p>
		<?php
	}

	/**
	 * Render block-cookies settings form part for admin page.
	 *
	 * @since       1.0.0
	 */
	public function render_block_cookies() {
		$options = get_option( 'etracker_settings' );
		?>
		<input type='checkbox' name='etracker_settings[etracker_block_cookies]' <?php checked( 'true', $options['etracker_block_cookies'] ); ?> value='true'>
		<p class="description">
		<?php esc_html_e( 'Set the default to disable cookies to comply with the EU regulations governing the use of cookies under the GDPR and the ePrivacy Directive.', 'etracker' ); ?>
		</p>
		<?php
	}

	/**
	 * Render disable_et_pagename settings form part for admin page.
	 *
	 * @since       1.3.0
	 */
	public function render_disable_et_pagename() {
		$options = get_option( 'etracker_settings' );
		?>
		<input type='checkbox' name='etracker_settings[etracker_disable_et_pagename]' <?php checked( 'true', $options['etracker_disable_et_pagename'] ); ?> value='true'>
		<p class="description">
		<?php esc_html_e( 'If enabled, automated page name registration setting from your etracker account will apply and no et_pagename option will be submitted. It is recommended to leave this option disabled.', 'etracker' ); ?>
		</p>
		<?php
	}

	/**
	 * Render signalize ready jQuery checker for admin page.
	 *
	 * @since       1.5.0
	 */
	public function render_signalize_ready() {
		?>
		<p id="etracker_signalize_ready_checked" style="display: none">✅</p>
		<p id="etracker_signalize_ready_unchecked" style="display: none">❌</p>
		<p class="description" id="etracker_signalize_ready_unknown">
		<?php esc_html_e( 'Signalize service worker integration will be checked in a moment.', 'etracker' ); ?>
		</p>
		<p class="description" id="etracker_signalize_ready_success" style="display: none">
		<?php esc_html_e( 'Your website is ready to use Signalize.', 'etracker' ); ?>
		</p>
		<p class="description" id="etracker_signalize_ready_failure" style="display: none">
		<?php esc_html_e( 'Your website is NOT ready to use Signalize. Try to disable and reenable the plugin etracker Analytics.', 'etracker' ); ?>
		</p>
		<script type='text/javascript'>
		jQuery(document).ready(function($) {
			$.ajax({
			type: 'GET',
			url: '/sw.js',
			dataType: 'text',
			success: function(data) {
				jQuery("#etracker_signalize_ready_unknown")[0].style.display = "none";
				jQuery("#etracker_signalize_ready_failure")[0].style.display = "none";
				jQuery("#etracker_signalize_ready_unchecked")[0].style.display = "none";
				jQuery("#etracker_signalize_ready_success")[0].style.display = null;
				jQuery("#etracker_signalize_ready_checked")[0].style.display = null;
			},
			error: function(xhr, status, error) {
				jQuery("#etracker_signalize_ready_unknown")[0].style.display = "none";
				jQuery("#etracker_signalize_ready_unchecked")[0].style.display = null;
				jQuery("#etracker_signalize_ready_failure")[0].style.display = null;
				jQuery("#etracker_signalize_ready_success")[0].style.display = "none";
				jQuery("#etracker_signalize_ready_checked")[0].style.display = "none";
			},
			});
		});
		</script>
		<?php
	}

	/**
	 * Render custom attributes settings form part for admin page.
	 *
	 * @since       1.7.0
	 */
	public function render_custom_attributes() {
		$options = get_option( 'etracker_settings' );
		?>
		<input size="75" type='text' name='etracker_settings[etracker_custom_attributes]' value='<?php echo esc_attr( $options['etracker_custom_attributes'] ); ?>'>
		<p class="description">
			<?php esc_html_e( 'Attention: Incorrect custom attributes can affect the tracking. Please have a look at our recommended attributes here:', 'etracker' ); ?>
			<?php $domain = __( 'https://www.etracker.com/en/docs/integration-setup-2/cms-shop-plugins/wordpress/', 'etracker' ); ?>
			<a href='<?php echo esc_url( $domain ); ?>' target="_blank"><?php esc_html_e( 'etracker WordPress Documentation.', 'etracker' ); ?></a>
		</p>
		<?php
	}

	/**
	 * Render settings link next to disable on plugin page.
	 *
	 * @param   array $links Action links to be filtered.
	 * @return  array $links
	 * @since       1.5.0
	 */
	public function add_settings_link( $links ) {
		// Build and escape the URL.
		$url = esc_url(
			add_query_arg(
				'page',
				'etracker',
				get_admin_url() . 'admin.php'
			)
		);
		// Create the link.
		$settings_link = "<a href='$url'>" . __( 'Settings', 'etracker' ) . '</a>';
		// Adds the link to the beginning of the array.
		array_unshift(
			$links,
			$settings_link
		);
		return $links;
	}

	/**
	 * Add rewrite rule to enable Signalize service worker.
	 *
	 * Adds rewrite rules required to provide Service Worker sw.js as DOC_ROOT.
	 *
	 * @param      string $wp_rewrite    WP_Rewrite object.
	 * @since    1.5.0
	 */
	public function generate_rewrite_rules( $wp_rewrite ) {
		// Add signalize rewrite rules.
		$wp_rewrite->add_external_rule( 'sw\.js$', 'wp-content/plugins/etracker/public/js/sw.js' );
	}
}
